const system = require("../../../system");
const ServiceBase = require("../../sve.base");
const settings = require("../../../../config/settings");
const appconfig = system.getSysConfig();
class AppService extends ServiceBase {
  constructor() {
    super("common", ServiceBase.getDaoName(AppService));
    this.userS = system.getObject("service.auth.userSve");
    this.routeDao=system.getObject("db.common.routeDao");
  }
  async getApp(appkey) {
    return this.cacheManager["AppCache"].cache(appkey, null);
  }
  async findAllApps(uid) {
    var apps=null;
    var dicRtn = {};
    var wheresql= {};
    if(uid){
      wheresql[this.db.Op.and]={
        [this.db.Op.or]:
          [
            {isPublish:false, creator_id: uid},
            {isEnabled: true,isPublish:true}
          ],
      };
      apps = await this.dao.model.findAll({ 
        where: wheresql, 
        attributes: ['id', 'name', 'appkey', 'showimgUrl', 'appType', 'docUrl','homePage'] });
    }else{
          wheresql= {isEnabled: true,isPublish:true};
          apps = await this.dao.model.findAll({ 
                  where: wheresql, 
                  attributes: ['id', 'name', 'appkey', 'showimgUrl', 'appType', 'docUrl','homePage'] });
    }
    for (var app of apps) {
      var tmk = uiconfig.config.pdict.app_type[app.appType];
      if (!dicRtn[tmk]) {
        dicRtn[tmk] = [];
        dicRtn[tmk].push(app);
      } else {
        dicRtn[tmk].push(app);
      }
    }
    return dicRtn;
  }
 //创建应用
 //每个应用建立两个路由，一个api路由
 //对api路由启用jwt插件
  async create(pobj, qobj, req) {
    var self = this;
    return this.db.transaction(async function (t) {
      var app = await self.dao.create(pobj, t);
      //创建后台应用服务
      let svobj=await self.cjsonregister(AppService.newServiceUrl(),{name:app.name,url:"http://"+app.backend})
      //添加路由
      let routeobj=await self.cjsonregister(AppService.newRouteUrl(app.name),
      {name:app.name,paths:["/web/auth/userCtl/pmlogin","/web/auth/userCtl/pmregister","/web/auth/userCtl/pmSendVCode","/web/auth/userCtl/pmloginByVCode"],hosts:[app.domainName],strip_path:false})
      let routeapi=await self.cjsonregister(AppService.newRouteUrl(app.name),{name:app.name+"_api",hosts:[app.domainName],paths:["/api","/web"],strip_path:false})
      let  r1=await  self.routeDao.create({name:app.name,center_id:routeobj.id,app_id:app.id},t);
      let  r2=await  self.routeDao.create({name:app.name+"_api",center_id:routeapi.id,app_id:app.id},t);
      //给api路由启动插件
      await self.cjsonregister(AppService.bindPluginUrl(app.name+"_api"),{name:"jwt"})
      if(svobj && routeobj && r1 && r2){
        try{
          app.appkey=svobj.id;
          await app.save({transaction:t});
        }catch(e){
          await self.cdel(AppService.routeUrl(app.name))
          await self.cdel(AppService.routeUrl(app.name+"_api"))
          await self.cdel(AppService.serviceUrl(app.name))
        }
      }else{
         throw new Error("创建应用服务失败");
      }
      return app;
    });
  }
  //删除应用
  async delete(pobj, qobj){
    var self=this;
    return this.db.transaction(async function (t) {
         await self.dao.delete(pobj,t)
         //删除路由
         await self.cdel(AppService.routeUrl(pobj.name));
         //删除api路由
         await self.cdel(AppService.routeUrl(pobj.name+"_api"));
         //删除服务
         await self.cdel(AppService.serviceUrl(pobj.name));
         return {}
    })
  }

  async findAndCountAll(obj) {
    var self = this;
    const apps = await super.findAndCountAll(obj);
    return apps;
  }
}
module.exports = AppService;
// (async ()=>{
//   let u=new AppService();
//  //  let x=await u.cregister("jiangong")
//  //  console.log(x)
//  // let x=await u.cunregister("jiangong")
//  //  console.log(x)
//   // let t=await u.cmakejwt()
//   // console.log(t)
//   //let ux=await u.cjsonregister(AppService.newRouteUrl("test-service2"),{name:"test-service2",hosts:["ttest1.com"]})
//   //let ux=await u.cjsonregister(AppService.newServiceUrl(),{name:"test-service3",url:"http://zhichan.gongsibao.com"})
//   //let ux=await u.cdel(AppService.routeUrl("test-service2"))
//   //let ux=await u.cdel(AppService.serviceUrl("test-service2"))
//   //  let ux=await u.create({name:"test4-service",backend:"zhichan-service",domainName:"domain.com"})
//   //   console.log(ux);
//   // let delrtn=await u.delete({id:2,name:"test4-service"})
//   // console.log(delrtn);
// })()